<?php

use CRM_Loginsecurity_ExtensionUtil as E;

class CRM_Loginsecurity_Form_Report_LoginSecurity extends CRM_Report_Form {
  protected $_addressField = TRUE;
  protected $_emailField = TRUE;
  protected $_noRepeats = FALSE;
  protected $_summary = NULL;
  protected $_customGroupExtends = ['Contact'];
  protected $_customGroupGroupBy = FALSE;

  function __construct() {
    $this->_columns = [
      'civicrm_contact' => [
        'dao' => 'CRM_Contact_DAO_Contact',
        'fields' => [
          'sort_name' => [
            'title' => E::ts('Contact Name'),
            'required' => TRUE,
            'default' => TRUE,
            'no_repeat' => TRUE,
          ],
          'id' => [
            'no_display' => TRUE,
            'required' => TRUE,
          ],
          'first_name' => [
            'title' => E::ts('First Name'),
            'no_repeat' => TRUE,
          ],
          'id' => [
            'no_display' => TRUE,
            'required' => TRUE,
          ],
          'last_name' => [
            'title' => E::ts('Last Name'),
            'no_repeat' => TRUE,
          ],
          'id' => [
            'no_display' => TRUE,
            'required' => TRUE,
          ],
        ],
        'filters' => [
          'sort_name' => [
            'title' => E::ts('Contact Name'),
            'operator' => 'like',
          ],
          'id' => [
            'no_display' => TRUE,
          ],
        ],
        'grouping' => 'contact-fields',
      ],
      'civicrm_login_security_device' => [
        'dao' => 'CRM_Loginsecurity_DAO_LoginSecurityDevice',
        'fields' => [
          'ip_address' => [
            'title' => 'IP Address',
            'default' => TRUE,
          ],
          'user_agent' => [
            'title' => E::ts('User Agent'),
            'default' => TRUE,
          ],
          'fingerprint' => [
            'title' => E::ts('Fingerprint'),
            'default' => TRUE,
          ],
          'ip_lookup' => [
            'title' => E::ts('IP Lookup'),
            'default' => TRUE,
            'pseudofield' => TRUE,
          ],
          'first_seen' => [
            'title' => E::ts('First Seen'),
            'default' => TRUE,
          ],
          'last_seen' => [
            'title' => E::ts('Last Seen'),
            'default' => TRUE,
          ],
        ],
        'filters' => [
          'first_seen' => [
            'title' => E::ts('First Seen'),
            'operatorType' => CRM_Report_Form::OP_DATE,
          ],
          'last_seen' => [
            'title' => E::ts('Last Seen'),
            'operatorType' => CRM_Report_Form::OP_DATE,
          ],
        ],
        'order_bys' => [
          'last_seen' => [
            'title' => ts('Last Seen'),
            'default' => '1',
            'default_weight' => '0',
            'default_order' => 'DESC',
          ],
          'first_seen' => [
            'title' => ts('First Seen'),
            'default' => '0',
            'default_weight' => '5',
            'default_order' => 'DESC',
          ],
        ],
        'grouping' => 'loginsecurity-fields',
      ],
      'civicrm_address' => [
        'dao' => 'CRM_Core_DAO_Address',
        'fields' => [
          'street_address' => NULL,
          'city' => NULL,
          'postal_code' => NULL,
          'state_province_id' => ['title' => E::ts('State/Province')],
          'country_id' => ['title' => E::ts('Country')],
        ],
        'grouping' => 'contact-fields',
      ],
      'civicrm_email' => [
        'dao' => 'CRM_Core_DAO_Email',
        'fields' => ['email' => NULL],
        'grouping' => 'contact-fields',
      ],
    ];
    $this->_groupFilter = TRUE;
    $this->_tagFilter = TRUE;
    parent::__construct();
  }

  function preProcess() {
    $this->assign('reportTitle', E::ts('Membership Detail Report'));
    parent::preProcess();
  }

  function from() {
    $this->_from = NULL;

    $this->_from = "
         FROM  civicrm_contact {$this->_aliases['civicrm_contact']} {$this->_aclFrom}
               INNER JOIN civicrm_login_security_device {$this->_aliases['civicrm_login_security_device']}
                          ON {$this->_aliases['civicrm_contact']}.id =
                             {$this->_aliases['civicrm_login_security_device']}.contact_id ";

    $this->joinAddressFromContact();
    $this->joinEmailFromContact();
  }

  /**
   * Add field specific select alterations.
   *
   * @param string $tableName
   * @param string $tableKey
   * @param string $fieldName
   * @param array $field
   *
   * @return string
   */
  function selectClause(&$tableName, $tableKey, &$fieldName, &$field) {
    return parent::selectClause($tableName, $tableKey, $fieldName, $field);
  }

  /**
   * Add field specific where alterations.
   *
   * This can be overridden in reports for special treatment of a field
   *
   * @param array $field Field specifications
   * @param string $op Query operator (not an exact match to sql)
   * @param mixed $value
   * @param float $min
   * @param float $max
   *
   * @return null|string
   */
  public function whereClause(&$field, $op, $value, $min, $max) {
    return parent::whereClause($field, $op, $value, $min, $max);
  }

  function alterDisplay(&$rows) {
    parent::alterDisplay($rows);

    foreach ($rows as &$row) {
      if (!empty($row['civicrm_login_security_device_ip_lookup'])) {
        $row['civicrm_login_security_device_ip_lookup'] = CRM_Loginsecurity_BAO_LoginSecurityDevice::getGeoIP($row['civicrm_login_security_device_ip_address'] ?? '');
      }
    }
  }

}
